#include <ESP8266WiFi.h>
#include <Ticker.h>
#include "Gsender.h"
#include <EEPROM.h>


Ticker timer_new;

const int sleepTime = 6; //Power down WiFi for 6 seconds

short test=true;

//gestione pin 
#define PIR_SENSOR            12
#define KEY_HW                13
#define BUZZER_LED_ON_12      14
#define  SIRENA               16 


// Inizializzo valori rete-wifi
const char* ssid     = "FuturaElettronica";
const char* password = "futurashop";

#define TEMPO_ABILITA_ALLARME     30 //30 secondi
#define TEMPO_DISABILITA_ALLARME  30 //30 secondi

#define CONNESSIONE      1
#define DISCONNESSIONE   2

void gestione_chiave_hw();
void gestione_pir();
short sound=false;

short is_connected=false;
short set_wifi=false;

uint8_t chiave_inserita = 0;
uint8_t old_chiave_inserita = 0;

uint8_t tentativi_invio=0;

int tempot=0;
short check_key=false;
int timer_key=0;

unsigned int timer_abilitazione_allarme=0;
unsigned int timer_disabilitazione_allarme=0;
unsigned int old_timer_abilitazione_allarme=0;
unsigned int old_timer_disabilitazione_allarme=0;

uint8_t timer_connect=0;
uint16_t timer_delay=0;
unsigned int old_tes_mov=0;
unsigned int tes_mov=0;

bool toggle = false;
bool abiita_antifurto=false;
bool movimento_rilevato = false;
bool invia_notifica = false;

bool t_wifi=true;

uint16_t addr_eeprom = 0;
uint16_t notifica_eeprom = 0;


void setup()
{
  Serial.begin(115200);
  delay(10);
  pinMode(SIRENA, OUTPUT);
  pinMode(KEY_HW, INPUT);   
  digitalWrite(KEY_HW, LOW);  
  pinMode(PIR_SENSOR, INPUT);       
  pinMode(BUZZER_LED_ON_12, OUTPUT);       
  digitalWrite(BUZZER_LED_ON_12,LOW);     
  digitalWrite(SIRENA,LOW);      
  EEPROM.begin(512);
  timer_new.attach_ms(1, timer0);

  
  abiita_antifurto=false;
  timer_abilitazione_allarme=TEMPO_ABILITA_ALLARME;  //carico tempo prima che si abiliti antifurto
  timer_disabilitazione_allarme=TEMPO_DISABILITA_ALLARME;
  invia_notifica=false;
  old_tes_mov=tes_mov+1;

  is_connected=true;  //ad ogni avvio faccio pulizia; (potrebbe esserci stato un watchdobg  



  gestione_wifi(DISCONNESSIONE);
  
  Serial.println("Attendere...");

  notifica_eeprom = EEPROM.read(addr_eeprom);  //leggo se ho da inviare allarmi

  if((notifica_eeprom!=0)&&(notifica_eeprom!=1)) //inizializzo valore in eeprom (caso indirizzo è a 0xFF)
  {
    notifica_eeprom=0;
    EEPROM.write(addr_eeprom, notifica_eeprom);
    EEPROM.commit();
    Serial.println("Indirizzo in eeprom inizializzato");
    delay(100);
  }

  if(notifica_eeprom==1)  //se leggo in memoria eeprom un allarme allora tento di inviarlo di nuovo
  {
    Serial.println("Attenzione notifica in memoria da inviare");
    invia_notifica=true;
  }
  
  Serial.println("Avvio programma");

  //collegare led a stato antifurto in modo da capire all'accensione se è attivo o meno

  chiave_inserita = digitalRead(KEY_HW);  // legge il valore dell'input e lo conserva
  old_chiave_inserita=chiave_inserita;    // all'avvio imposto lo stesso stato cosi è l'utente che decide il cambiamento
  tentativi_invio=3;
  digitalWrite(BUZZER_LED_ON_12,LOW); 
  digitalWrite(SIRENA,LOW);
}
void loop() {
  
  gestione_chiave_hw();
  gestione_pir();  


  if(old_timer_abilitazione_allarme!=timer_abilitazione_allarme)
  {
    old_timer_abilitazione_allarme=timer_abilitazione_allarme;
    Serial.print("Timer abilita: ");        //
    Serial.println(timer_abilitazione_allarme);        //
  }
  if(old_timer_disabilitazione_allarme!=timer_disabilitazione_allarme)
  {
    old_timer_disabilitazione_allarme=timer_disabilitazione_allarme;
    Serial.print("Timer disabilita: ");        //
    Serial.println(timer_disabilitazione_allarme);        //
  }

  if((timer_disabilitazione_allarme==false)||(notifica_eeprom==1))
  {
    if(invia_notifica)
    {
      Serial.println("inizio gestione connessione!");
      if(gestione_wifi(CONNESSIONE))  //se riesco a connettermi entro i secondi impostati
      {
        Serial.println("Invio notifica allarme"); 
        Gsender *gsender = Gsender::Instance();    // Getting pointer to class instance
        String subject = "ANTIFURTO CASA: ";
        
        //questo if si occupa di inviare la mail di notifica -> inserire solo indirizzo mail
        if((gsender->Subject(subject)->Send("gcavallaro2@alice.it", "Attenzione movimento rilevato"))&&(tentativi_invio>0)) 
        {
            Serial.println("Messagio inviato correttamente.");
            Serial.println("gestione disconnessione init!");
            gestione_wifi(DISCONNESSIONE);
            Serial.println("gestione disconnessione end!");
            invia_notifica=false;

            if(notifica_eeprom==1) //se ho inviato allarme salvato in memoria pulisco la eeprom
            {
              notifica_eeprom=0;
              EEPROM.write(addr_eeprom, notifica_eeprom);
              EEPROM.commit();
              delay(100);
              Serial.println("Cancello notifiche in eeprom");
            }
        } 
        else 
        {
            tentativi_invio--;
            
            if(tentativi_invio==0)
            {
              gestione_wifi(DISCONNESSIONE);
              Serial.println("gestione disconnessione end!");
              invia_notifica=false;
            }
            Serial.print("Errore invio messaggio: ");
            Serial.println(gsender->getError());
            Serial.print("Nuovo tentativo");

        }  
      }
      else
      {
        //non sono riuscito a connettermi
        Serial.println("Errore connessione Wi-fi");
        //salvo valore allarme in memoria
        if(notifica_eeprom==0)
        { 
            notifica_eeprom=1;  //salvo lo stato invio notifica e resetto modulo
            EEPROM.write(addr_eeprom, notifica_eeprom);
            EEPROM.commit();
            delay(100);
            Serial.println("Salvataggio notifica in eeprom");
            //eseguo reset modulo esp
        }
        invia_notifica=false; 
        Serial.println("Reset modulo");
        delay(100);
        ESP.restart();
         
      }
    }
    
  }
  
}
